package cn.iocoder.yudao.module.order.service.orderSplitItem;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.order.vo.orderSplitItem.*;
import cn.iocoder.yudao.module.order.dal.dataobject.orderSplitItem.OrderSplitItemDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 订单拆单产品项 Service 接口
 *
 * @author yanghao
 */
public interface OrderSplitItemService extends IService<OrderSplitItemDO> {

    /**
     * 创建订单拆单产品项
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Boolean createSplitItem(@Valid OrderSplitItemCreateReqVO createReqVO);


    /**
     * 校验订单计价单位是否为(个)，为(个)则需要传数量值 quantity
     *
     * @param freightVolume          运费计量单位ID
     * @param clearanceFreightVolume 清关费计量单位ID
     * @return 是/否
     */
    Boolean checkItemVolume(Integer freightVolume, Integer clearanceFreightVolume);

    /**
     * 更新订单拆单产品项
     *
     * @param updateReqVO 更新信息
     */
    Boolean updateSplitItem(@Valid OrderSplitItemUpdateReqVO updateReqVO);

    /**
     * 删除订单拆单产品项
     *
     * @param id 编号
     * @param shipmentSplit 是否出货拆单
     */
    void deleteSplitItem(Long id, Boolean shipmentSplit);

    /**
     * 获得订单拆单产品项
     *
     * @param id 编号
     * @return 订单拆单产品项
     */
    OrderSplitItemDO getSplitItem(Long id);

    /**
     * 获得订单拆单产品项列表
     *
     * @param ids 编号
     * @return 订单拆单产品项列表
     */
    List<OrderSplitItemDO> getSplitItemList(Collection<Long> ids);

    /**
     * 获得订单拆单产品项分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 订单拆单产品项分页
     */
    PageResult<OrderSplitItemDO> getSplitItemPage(OrderSplitItemQueryVO query, PageVO page);

    /**
     * 获得订单拆单产品项列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 订单拆单产品项列表
     */
    List<OrderSplitItemDO> getSplitItemList(OrderSplitItemQueryVO query);

    Boolean batchCreate(OrderSplitItemBatchCreateReqVO createReqVO);
}
